@@ -99,6 +99,62 @@ class SQLLogicParser:
             "<integral>": ["tinyint", "smallint", "integer", "bigint", "hugeint"],
             "<signed>": ["tinyint", "smallint", "integer", "bigint", "hugeint"],
             "<unsigned>": ["utinyint", "usmallint", "uinteger", "ubigint", "uhugeint"],
+            "<unsigned>": ["utinyint", "usmallint", "uinteger", "ubigint", "uhugeint"],
+            "<all_types_columns>": [
+                "bool",
+                "tinyint",
+                "smallint",
+                "int",
+                "bigint",
+                "hugeint",
+                "uhugeint",
+                "utinyint",
+                "usmallint",
+                "uint",
+                "ubigint",
+                "date",
+                "time",
+                "timestamp",
+                "timestamp_s",
+                "timestamp_ms",
+                "timestamp_ns",
+                "time_tz",
+                "timestamp_tz",
+                "float",
+                "double",
+                "dec_4_1",
+                "dec_9_4",
+                "dec_18_6",
+                "dec38_10",
+                "uuid",
+                "interval",
+                "varchar",
+                "blob",
+                "bit",
+                "small_enum",
+                "medium_enum",
+                "large_enum",
+                "int_array",
+                "double_array",
+                "date_array",
+                "timestamp_array",
+                "timestamptz_array",
+                "varchar_array",
+                "nested_int_array",
+                "struct",
+                "struct_of_arrays",
+                "array_of_structs",
+                "map",
+                "union",
+                "fixed_int_array",
+                "fixed_varchar_array",
+                "fixed_nested_int_array",
+                "fixed_nested_varchar_array",
+                "fixed_struct_array",
+                "struct_of_fixed_array",
+                "fixed_array_of_int_list",
+                "list_of_fixed_int_array",
+            ],
         }
 
     def peek(self):
